@@ -22,13 +22,13 @@ use std::{
 
 use bitflags::bitflags;
 use libseccomp::ScmpSyscall;
-use memchr::arch::all::is_equal;
 use nix::{
     errno::Errno,
     fcntl::{AtFlags, OFlag},
     sys::{
         epoll::EpollOp,
         socket::SockaddrLike,
+        stat::makedev,
         wait::{Id, WaitPidFlag, WaitStatus as NixWaitStatus},
     },
     unistd::Pid,
@@ -36,7 +36,19 @@ use nix::{
 };
 use serde::{ser::SerializeMap, Serialize, Serializer};
 
-use crate::{config::*, fs::FileType, XPath};
+use crate::{config::*, lookup::FileType, XPath};
+
+pub(crate) const UIO_MAXIOV: usize = libc::UIO_MAXIOV as usize;
+pub(crate) const UIO_MAXLEN: usize = 256_000;
+
+// x32 compatibility
+// See https://sourceware.org/bugzilla/show_bug.cgi?id=16437
+#[cfg(all(target_arch = "x86_64", target_pointer_width = "32"))]
+#[expect(non_camel_case_types)]
+pub(crate) type timespec_tv_nsec_t = i64;
+#[cfg(not(all(target_arch = "x86_64", target_pointer_width = "32")))]
+#[expect(non_camel_case_types)]
+pub(crate) type timespec_tv_nsec_t = libc::c_long;
 
 // C-compatible layout of the `timespec64` structure.
 #[derive(Clone, Copy, Debug, Eq, Hash, PartialEq)]
@@ -83,7 +95,7 @@ pub(crate) const fn cmsg_align_32(len: usize) -> usize {
 ///
 /// The total space in bytes required to store the control message, ensuring proper alignment.
 pub(crate) const fn cmsg_space_32(length: u32) -> usize {
-    cmsg_align_32((length as usize).saturating_add(cmsg_align_32(std::mem::size_of::<cmsghdr32>())))
+    cmsg_align_32((length as usize).saturating_add(cmsg_align_32(size_of::<cmsghdr32>())))
 }
 
 /// Computes the byte length of a control message's header and data for a 32-bit system.
@@ -99,7 +111,7 @@ pub(crate) const fn cmsg_space_32(length: u32) -> usize {
 ///
 /// The combined length of the control message header and the data, aligned as required.
 pub(crate) const fn cmsg_len_32(length: u32) -> usize {
-    cmsg_align_32(std::mem::size_of::<cmsghdr32>()).saturating_add(length as usize)
+    cmsg_align_32(size_of::<cmsghdr32>()).saturating_add(length as usize)
 }
 
 /// This structure represents the Linux 32-bit data structure 'struct stat'
@@ -188,6 +200,7 @@ impl From<libc::stat64> for stat32 {
 }
 
 /// This structure represents the Linux 32-bit data structure 'struct iovec'
+#[derive(Copy, Clone)]
 #[repr(C)]
 pub struct iovec32 {
     iov_base: u32,
@@ -237,6 +250,52 @@ impl From<mmsghdr> for mmsghdr32 {
     }
 }
 
+const _: () = {
+    assert!(
+        size_of::<libc::mmsghdr>() == size_of::<mmsghdr>(),
+        "Size mismatch between libc::mmsghdr and compat::mmsghdr"
+    );
+};
+
+impl From<libc::mmsghdr> for mmsghdr {
+    fn from(msg: libc::mmsghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::mmsghdr and mmsghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<libc::mmsghdr> for mmsghdr32 {
+    fn from(msg: libc::mmsghdr) -> Self {
+        mmsghdr::from(msg).into()
+    }
+}
+
+impl From<mmsghdr> for libc::mmsghdr {
+    fn from(msg: mmsghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::mmsghdr and mmsghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<mmsghdr32> for libc::mmsghdr {
+    fn from(msg: mmsghdr32) -> Self {
+        mmsghdr::from(msg).into()
+    }
+}
+
+/// This union represents the Linux data structure 'struct mmsghdr'
+#[repr(C)]
+pub union mmsghdr_union {
+    // 32-bit mmsghdr32
+    pub m32: mmsghdr32,
+    // Native mmsghdr
+    pub m64: mmsghdr,
+}
+
 /// This structure represents the Linux 32-bit data structure 'struct cmsghdr'
 #[repr(C)]
 pub struct cmsghdr32 {
@@ -320,6 +379,28 @@ impl From<msghdr> for msghdr32 {
     }
 }
 
+const _: () = {
+    assert!(
+        size_of::<libc::msghdr>() == size_of::<msghdr>(),
+        "Size mismatch between libc::msghdr and compat::msghdr"
+    );
+};
+
+impl From<libc::msghdr> for msghdr {
+    fn from(msg: libc::msghdr) -> Self {
+        // SAFETY: Since we have confirmed the sizes are the same at compile time,
+        // we can safely perform a transmute. This assumes no differences in alignment
+        // or memory layout between libc::msghdr and msghdr.
+        unsafe { std::mem::transmute(msg) }
+    }
+}
+
+impl From<libc::msghdr> for msghdr32 {
+    fn from(msg: libc::msghdr) -> Self {
+        msghdr::from(msg).into()
+    }
+}
+
 /// Rust equivalent of the Linux kernel's struct xattr_args:
 #[repr(C)]
 #[derive(Debug, Copy, Clone)]
@@ -378,7 +459,7 @@ pub unsafe fn getxattrat<Fd: AsFd, P: ?Sized + NixPath>(
                     flags.bits(),
                     name,
                     args as *mut XattrArgs,
-                    std::mem::size_of::<XattrArgs>(),
+                    size_of::<XattrArgs>(),
                 )
             },
         )
@@ -426,7 +507,7 @@ pub unsafe fn setxattrat<Fd: AsFd, P: ?Sized + NixPath>(
                 flags.bits(),
                 name,
                 args as *const XattrArgs,
-                std::mem::size_of::<XattrArgs>(),
+                size_of::<XattrArgs>(),
             )
         })
         .map(drop)
@@ -533,7 +614,7 @@ pub unsafe fn removexattrat<Fd: AsFd, P: ?Sized + NixPath>(
 pub struct FileStatxTimestamp {
     pub tv_sec: i64,
     pub tv_nsec: u32,
-    __statx_timestamp_pad1: [i32; 1],
+    pub(crate) __statx_timestamp_pad1: [i32; 1],
 }
 
 impl PartialEq for FileStatxTimestamp {
@@ -682,7 +763,7 @@ pub struct DirIter {
     offset: usize,
 }
 
-const DIRENT_ALIGN: usize = std::mem::align_of::<libc::dirent64>();
+const DIRENT_ALIGN: usize = align_of::<libc::dirent64>();
 
 impl DirIter {
     /// Creates a new `DirIter` with an allocated buffer of `bufsiz` bytes.
@@ -818,14 +899,14 @@ impl DirEntry<'_> {
         self.dirent
     }
 
-    /// Returns true if this is a dot entry.
-    /// The special dot entries are `.` and `..`
+    /// Returns true if this is the `.` (dot) entry.
     pub fn is_dot(&self) -> bool {
-        if !self.is_dir() {
-            return false;
-        }
-        let name = self.name_bytes();
-        is_equal(name, b"..") || is_equal(name, b".")
+        self.is_dir() && self.as_xpath().is_equal(b".")
+    }
+
+    /// Returns true if this is the `..` (dotdot) entry.
+    pub fn is_dotdot(&self) -> bool {
+        self.is_dir() && self.as_xpath().is_equal(b"..")
     }
 
     /// Returns true if this is a directory entry.
@@ -1220,16 +1301,23 @@ pub const AT_STATX_FORCE_SYNC: libc::c_int = 0x2000;
 /// Do not sync with remote filesystem.
 pub const AT_STATX_DONT_SYNC: libc::c_int = 0x4000;
 
-/// Safe statx() wrapper.
+/// Safe statx(2) wrapper.
+///
+/// This function sets the flag AT_STATX_DONT_SYNC if AT_STATX_FORCE_SYNC is not set.
 pub fn statx<Fd: AsFd, P: ?Sized + NixPath>(
     dirfd: Fd,
     pathname: &P,
-    flags: libc::c_int,
+    mut flags: libc::c_int,
     mask: libc::c_uint,
 ) -> Result<FileStatx, Errno> {
     let dirfd = dirfd.as_fd().as_raw_fd();
     let mut dst = std::mem::MaybeUninit::uninit();
 
+    // SAFETY: Force DONT_SYNC if FORCE_SYNC is not set.
+    if flags & AT_STATX_FORCE_SYNC == 0 {
+        flags |= AT_STATX_DONT_SYNC;
+    }
+
     // SAFETY: Neither nix nor libc has a wrapper for statx.
     Errno::result(pathname.with_nix_path(|cstr| unsafe {
         libc::syscall(
@@ -1246,7 +1334,9 @@ pub fn statx<Fd: AsFd, P: ?Sized + NixPath>(
     Ok(unsafe { dst.assume_init() })
 }
 
-/// Safe statx() wrapper to use with a FD only.
+/// Safe statx(2) wrapper to use with a FD only.
+///
+/// This function always sets the flag AT_STATX_DONT_SYNC.
 pub fn fstatx<Fd: AsFd>(fd: Fd, mask: libc::c_uint) -> Result<FileStatx, Errno> {
     let fd = fd.as_fd().as_raw_fd();
     let mut dst = std::mem::MaybeUninit::uninit();
@@ -1257,7 +1347,7 @@ pub fn fstatx<Fd: AsFd>(fd: Fd, mask: libc::c_uint) -> Result<FileStatx, Errno>
             libc::SYS_statx,
             fd,
             c"".as_ptr(),
-            libc::AT_EMPTY_PATH,
+            libc::AT_EMPTY_PATH | AT_STATX_DONT_SYNC,
             mask,
             dst.as_mut_ptr(),
         )
@@ -1270,21 +1360,53 @@ pub fn fstatx<Fd: AsFd>(fd: Fd, mask: libc::c_uint) -> Result<FileStatx, Errno>
 /// Wrapper for struct stat64.
 pub(crate) use libc::stat64 as FileStat64;
 
-pub(crate) fn fstatat64<P: ?Sized + NixPath>(
-    dirfd: Option<RawFd>,
+impl From<FileStatx> for FileStat64 {
+    fn from(stx: FileStatx) -> FileStat64 {
+        FileStat64::from(&stx)
+    }
+}
+
+impl From<&FileStatx> for FileStat64 {
+    #[expect(clippy::as_underscore)]
+    #[expect(clippy::cast_lossless)]
+    #[expect(clippy::cast_possible_wrap)]
+    fn from(stx: &FileStatx) -> FileStat64 {
+        // SAFETY: FileStat64 is a POD struct.
+        let mut st: FileStat64 = unsafe { std::mem::zeroed() };
+
+        st.st_ino = stx.stx_ino;
+        st.st_nlink = stx.stx_nlink.into();
+        st.st_mode = stx.stx_mode.into();
+        st.st_uid = stx.stx_uid;
+        st.st_gid = stx.stx_gid;
+        st.st_size = stx.stx_size as i64;
+        st.st_blksize = stx.stx_blksize as _;
+        st.st_blocks = stx.stx_blocks as i64;
+        st.st_atime = stx.stx_atime.tv_sec as _;
+        st.st_atime_nsec = stx.stx_atime.tv_nsec as _;
+        st.st_mtime = stx.stx_mtime.tv_sec as _;
+        st.st_mtime_nsec = stx.stx_mtime.tv_nsec as _;
+        st.st_ctime = stx.stx_ctime.tv_sec as _;
+        st.st_ctime_nsec = stx.stx_ctime.tv_nsec as _;
+
+        st.st_dev = makedev(stx.stx_dev_major.into(), stx.stx_dev_minor.into());
+        st.st_rdev = makedev(stx.stx_rdev_major.into(), stx.stx_rdev_minor.into());
+
+        st
+    }
+}
+
+pub(crate) fn fstatat64<Fd: AsFd, P: ?Sized + NixPath>(
+    dirfd: Fd,
     pathname: &P,
     flags: libc::c_int,
 ) -> Result<FileStat64, Errno> {
-    let dirfd = dirfd.unwrap_or(libc::AT_FDCWD);
-    let mut dst = std::mem::MaybeUninit::uninit();
-
-    // SAFETY: nix does not has a wrapper for fstatat64.
-    Errno::result(pathname.with_nix_path(|cstr| unsafe {
-        libc::fstatat64(dirfd, cstr.as_ptr(), dst.as_mut_ptr(), flags)
-    })?)?;
-
-    // SAFETY: fstatat64 returned success.
-    Ok(unsafe { dst.assume_init() })
+    const FLAGS: libc::c_int =
+        libc::AT_EMPTY_PATH | libc::AT_NO_AUTOMOUNT | libc::AT_SYMLINK_NOFOLLOW;
+    if flags & !FLAGS != 0 {
+        return Err(Errno::EINVAL);
+    }
+    Ok(statx(dirfd, pathname, flags, STATX_BASIC_STATS)?.into())
 }
 
 #[cfg(target_os = "freebsd")]
@@ -1311,7 +1433,9 @@ type fs_type_t = libc::c_int;
 type fs_type_t = libc::__fsword_t;
 
 const BTRFS_SUPER_MAGIC: fs_type_t = libc::BTRFS_SUPER_MAGIC as fs_type_t;
+const HUGETLBFS_MAGIC: fs_type_t = libc::HUGETLBFS_MAGIC as fs_type_t;
 const OVERLAYFS_SUPER_MAGIC: fs_type_t = libc::OVERLAYFS_SUPER_MAGIC as fs_type_t;
+const PROC_SUPER_MAGIC: fs_type_t = libc::PROC_SUPER_MAGIC as fs_type_t;
 
 /// Wrapper for struct statfs64
 pub(crate) struct Statfs64(libc::statfs64);
@@ -1328,6 +1452,16 @@ impl Statfs64 {
     pub(crate) fn has_broken_device_ids(&self) -> bool {
         matches!(self.0.f_type, OVERLAYFS_SUPER_MAGIC | BTRFS_SUPER_MAGIC)
     }
+
+    // Check if file resides on a hugetlbfs.
+    pub(crate) fn is_huge_file(&self) -> bool {
+        self.0.f_type == HUGETLBFS_MAGIC
+    }
+
+    // Check if file resides on a procfs.
+    pub(crate) fn is_proc(&self) -> bool {
+        self.0.f_type == PROC_SUPER_MAGIC
+    }
 }
 
 /// Safe wrapper for fstatfs64
@@ -1488,7 +1622,7 @@ pub fn epoll_get_params<Fd: AsFd>(fd: Fd) -> Result<EpollParams, Errno> {
 /// Uses getsockopt SO_DOMAIN to get the domain of the given socket.
 pub fn getsockdomain<Fd: AsFd>(fd: Fd) -> Result<libc::c_int, Errno> {
     #[expect(clippy::cast_possible_truncation)]
-    let mut len = std::mem::size_of::<libc::c_int>() as libc::socklen_t;
+    let mut len = size_of::<libc::c_int>() as libc::socklen_t;
     let mut fml: libc::c_int = 0;
 
     // SAFETY: In libc we trust.
@@ -1648,7 +1782,7 @@ bitflags! {
     #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
     #[repr(transparent)]
     pub struct RenameFlags: u32 {
-        /// Don’t overwrite `new_path` of the rename;
+        /// Don't overwrite `new_path` of the rename;
         /// return an error if `new_path` already exists.
         const RENAME_NOREPLACE = 1;
 
@@ -1656,8 +1790,263 @@ bitflags! {
         /// Both paths must exist.
         const RENAME_EXCHANGE  = 2;
 
-        /// Create a “whiteout” at the source of the rename
+        /// Create a "whiteout" at the source of the rename
         /// (for overlay/union filesystems).
         const RENAME_WHITEOUT  = 4;
     }
 }
+
+bitflags! {
+    /// Flags used with `send`, `recv`, etc.
+    // Keep in sync with <linux/socket.h>!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct MsgFlags: i32 {
+        /// Process out-of-band data.
+        const MSG_OOB = 0x01;
+        /// Peek at incoming messages.
+        const MSG_PEEK = 0x02;
+        /// Don't use local routing.
+        const MSG_DONTROUTE = 0x04;
+        /// DECnet uses a different name for MSG_DONTROUTE.
+        const MSG_TRYHARD = Self::MSG_DONTROUTE.bits();
+        /// Control data lost before delivery.
+        const MSG_CTRUNC = 0x08;
+        /// Supply or ask second address.
+        const MSG_PROXY = 0x10;
+        /// Truncated message.
+        const MSG_TRUNC = 0x20;
+        /// Nonblocking IO.
+        const MSG_DONTWAIT = 0x40;
+        /// End of record.
+        const MSG_EOR = 0x80;
+        /// Wait for a full request.
+        const MSG_WAITALL = 0x100;
+        /// End of connection.
+        const MSG_FIN = 0x200;
+        /// Synchronous operation.
+        const MSG_SYN = 0x400;
+        /// Confirm path validity.
+        const MSG_CONFIRM = 0x800;
+        /// Reset connection.
+        const MSG_RST = 0x1000;
+        /// Fetch message from error queue.
+        const MSG_ERRQUEUE = 0x2000;
+        /// Do not generate SIGPIPE.
+        const MSG_NOSIGNAL = 0x4000;
+        /// Sender will send more data.
+        const MSG_MORE = 0x8000;
+        /// Wait for at least one packet to return.
+        const MSG_WAITFORONE = 0x10000;
+        /// More messages coming.
+        const MSG_BATCH = 0x40000;
+        /// Receive devmem skbs as cmsg.
+        const MSG_SOCK_DEVMEM = 0x2000000;
+        /// Use user data in kernel path.
+        const MSG_ZEROCOPY = 0x4000000;
+        /// Send data in TCP SYN.
+        const MSG_FASTOPEN = 0x20000000;
+        /// Set close_on_exit for file descriptors.
+        const MSG_CMSG_CLOEXEC = 0x40000000;
+        /// Special flag for notifications.
+        const MSG_NOTIFICATION = Self::MSG_MORE.bits();
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::socket::MsgFlags as NixMsgFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<MsgFlags> for NixMsgFlags {
+    fn from(msgflags: MsgFlags) -> Self {
+        Self::from_bits_retain(msgflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixMsgFlags> for MsgFlags {
+    fn from(msgflags: NixMsgFlags) -> Self {
+        Self::from_bits_retain(msgflags.bits())
+    }
+}
+
+bitflags! {
+    /// Flags used with inotify_add_watch(2).
+    // Keep in sync with <linux/inotify.h>!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct AddWatchFlags: u32 {
+        // The following are legal, implemented events that user-space can watch for.
+        /// File was accessed.
+        const IN_ACCESS = 0x00000001;
+        /// File was modified.
+        const IN_MODIFY = 0x00000002;
+        /// Metadata changed.
+        const IN_ATTRIB = 0x00000004;
+        /// Writable file was closed.
+        const IN_CLOSE_WRITE = 0x00000008;
+        /// Unwritable file closed.
+        const IN_CLOSE_NOWRITE = 0x00000010;
+        /// File was opened.
+        const IN_OPEN = 0x00000020;
+        /// File was moved from X.
+        const IN_MOVED_FROM = 0x00000040;
+        /// File was moved to Y.
+        const IN_MOVED_TO = 0x00000080;
+        /// Subfile was created.
+        const IN_CREATE = 0x00000100;
+        /// Subfile was deleted.
+        const IN_DELETE = 0x00000200;
+        /// Self was deleted.
+        const IN_DELETE_SELF = 0x00000400;
+        /// Self was moved.
+        const IN_MOVE_SELF = 0x00000800;
+
+        // The following are legal events. They are sent as needed to any watch.
+        /// Backing fs was unmounted.
+        const IN_UNMOUNT = 0x00002000;
+        /// Event queued overflowed.
+        const IN_Q_OVERFLOW = 0x00004000;
+        /// File was ignored.
+        const IN_IGNORED = 0x00008000;
+
+        // Helper events.
+        /// Helper event: Close.
+        const IN_CLOSE = Self::IN_CLOSE_WRITE.bits() | Self::IN_CLOSE_NOWRITE.bits();
+        /// Helper event: Moves.
+        const IN_MOVE = Self::IN_MOVED_FROM.bits() | Self::IN_MOVED_TO.bits();
+
+        // Special flags.
+        /// Only watch the path if it is a directory.
+        const IN_ONLYDIR = 0x01000000;
+        /// Don't follow a sym link.
+        const IN_DONT_FOLLOW = 0x02000000;
+        /// Exclude events on unlinked objects.
+        const IN_EXCL_UNLINK = 0x04000000;
+        /// Only create watches.
+        const IN_MASK_CREATE = 0x10000000;
+        /// Add to the mask of an already existing watch.
+        const IN_MASK_ADD = 0x20000000;
+        /// Event occurred against dir.
+        const IN_ISDIR = 0x40000000;
+        /// Only send event once.
+        const IN_ONESHOT = 0x80000000;
+
+        /*
+         * All of the events - we build the list by hand so that we can add flags in
+         * the future and not break backward compatibility. Apps will get only the
+         * events that they originally wanted. Be sure to add new events here!
+         */
+        const IN_ALL_EVENTS =
+            Self::IN_ACCESS.bits() |
+            Self::IN_MODIFY.bits() |
+            Self::IN_ATTRIB.bits() |
+            Self::IN_CLOSE_WRITE.bits() |
+            Self::IN_CLOSE_NOWRITE.bits() |
+            Self::IN_OPEN.bits() |
+            Self::IN_MOVED_FROM.bits() |
+            Self::IN_MOVED_TO.bits() |
+            Self::IN_DELETE.bits() |
+            Self::IN_CREATE.bits() |
+            Self::IN_DELETE_SELF.bits() |
+            Self::IN_MOVE_SELF.bits();
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::inotify::AddWatchFlags as NixAddWatchFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<AddWatchFlags> for NixAddWatchFlags {
+    fn from(addwatchflags: AddWatchFlags) -> Self {
+        Self::from_bits_retain(addwatchflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixAddWatchFlags> for AddWatchFlags {
+    fn from(addwatchflags: NixAddWatchFlags) -> Self {
+        Self::from_bits_retain(addwatchflags.bits())
+    }
+}
+
+bitflags! {
+    /// Flags for memfd_create(2)
+    // nix' MFdFlags does not include MFD_{EXEC,NOEXEC_SEAL} yet!
+    #[derive(Copy, Clone, Debug, Eq, Hash, Ord, PartialEq, PartialOrd)]
+    #[repr(transparent)]
+    pub struct MFdFlags: libc::c_uint {
+        /// Close-on-exec
+        const MFD_CLOEXEC = libc::MFD_CLOEXEC;
+
+        /// Allow sealing via _fcntl_(2).
+        const MFD_ALLOW_SEALING = libc::MFD_ALLOW_SEALING;
+
+        /// Disallow exec (Linux >= 6.3).
+        const MFD_NOEXEC_SEAL = libc::MFD_NOEXEC_SEAL;
+
+        /// Allow exec (Linux >= 6.3).
+        const MFD_EXEC = libc::MFD_EXEC;
+
+        /// Use hugetlbfs.
+        const MFD_HUGETLB = libc::MFD_HUGETLB;
+
+        /// Huge page size: 1MB.
+        const MFD_HUGE_1MB = libc::MFD_HUGE_1MB;
+
+        /// Huge page size: 2MB.
+        const MFD_HUGE_2MB = libc::MFD_HUGE_2MB;
+
+        /// Huge page size: 8MB.
+        const MFD_HUGE_8MB = libc::MFD_HUGE_8MB;
+
+        /// Huge page size: 16MB.
+        const MFD_HUGE_16MB = libc::MFD_HUGE_16MB;
+
+        /// Huge page size: 32MB.
+        const MFD_HUGE_32MB = libc::MFD_HUGE_32MB;
+
+        /// Huge page size: 256MB.
+        const MFD_HUGE_256MB = libc::MFD_HUGE_256MB;
+
+        /// Huge page size: 512MB.
+        const MFD_HUGE_512MB = libc::MFD_HUGE_512MB;
+
+        /// Huge page size: 1GB.
+        const MFD_HUGE_1GB = libc::MFD_HUGE_1GB;
+
+        /// Huge page size: 2GB.
+        const MFD_HUGE_2GB = libc::MFD_HUGE_2GB;
+
+        /// Huge page size: 16GB.
+        const MFD_HUGE_16GB = libc::MFD_HUGE_16GB;
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+use nix::sys::memfd::MFdFlags as NixMFdFlags;
+
+#[expect(clippy::disallowed_types)]
+impl From<MFdFlags> for NixMFdFlags {
+    fn from(mfdflags: MFdFlags) -> Self {
+        Self::from_bits_retain(mfdflags.bits())
+    }
+}
+
+#[expect(clippy::disallowed_types)]
+impl From<NixMFdFlags> for MFdFlags {
+    fn from(mfdflags: NixMFdFlags) -> Self {
+        Self::from_bits_retain(mfdflags.bits())
+    }
+}
+
+/*
+ * Constants from <linux/limits.h> not defined by libc yet.
+ */
+
+// # chars in an extended attribute name.
+pub(crate) const XATTR_NAME_MAX: usize = 255;
+// size of an extended attribute value (64k).
+pub(crate) const XATTR_SIZE_MAX: usize = 1 << 16;
+// size of extended attribute namelist (64k).
+pub(crate) const XATTR_LIST_MAX: usize = 1 << 16;
