@@ -8,6 +8,7 @@
 use std::{
     ffi::CStr,
     fmt::Display,
+    fs::{exists, read_to_string},
     os::{
         fd::{AsFd, AsRawFd, RawFd},
         unix::process::ExitStatusExt,
@@ -52,9 +53,9 @@ use crate::{
         Scope, ABI,
     },
     lookup::FileType,
-    path::{mask_path, XPathBuf},
+    path::{mask_path, XPath, XPathBuf},
     retry::retry_on_eintr,
-    sandbox::{IoctlMap, Sandbox},
+    sandbox::{RawIoctlMap, Sandbox},
 };
 
 // Used as dummy errno(3) after SECCOMP_IOCTL_NOTIF_ADDFD with SECCOMP_ADDFD_FLAG_SEND flag.
@@ -200,7 +201,7 @@ pub fn confine_scmp_wx() -> SydResult<()> {
 /// Add per-architecture seccomp(2) filters to deny given ioctl(2) requests.
 ///
 /// Set `ssb` to true to disable Speculative Store Bypass mitigations.
-pub fn confine_scmp_ioctl(denylist: &IoctlMap, ssb: bool) -> SydResult<()> {
+pub fn confine_scmp_ioctl(denylist: &RawIoctlMap, ssb: bool) -> SydResult<()> {
     let syscall = ScmpSyscall::from_name("ioctl").or(Err(Errno::ENOSYS))?;
     for arch in SCMP_ARCH {
         let denylist = if let Some(denylist) = denylist.get(arch) {
@@ -2185,6 +2186,42 @@ pub(crate) fn is_coredump(sig: i32) -> bool {
     )
 }
 
+/// Returns `Ok(true)` if SELinux is enabled.
+#[expect(clippy::disallowed_methods)]
+pub fn selinux_enabled() -> Result<bool, Errno> {
+    let data = read_to_string("/proc/thread-self/mounts").map_err(|err| err2no(&err))?;
+    let data = XPath::from_bytes(data.as_bytes());
+    Ok(data.contains(b"selinux"))
+}
+
+/// Returns `Ok(true)` if AppArmor is enabled.
+#[expect(clippy::disallowed_methods)]
+pub fn apparmor_enabled() -> Result<bool, Errno> {
+    if !exists("/sys/kernel/security/apparmor").map_err(|err| err2no(&err))? {
+        return Ok(false);
+    }
+    let data =
+        read_to_string("/sys/module/apparmor/parameters/enabled").map_err(|err| err2no(&err))?;
+    Ok(data.as_bytes().first() == Some(&b'Y'))
+}
+
+/// Returns `Ok(true)` if SELinux is enforced.
+#[expect(clippy::disallowed_methods)]
+pub fn selinux_enforced() -> Result<bool, Errno> {
+    let path = if exists("/sys/fs/selinux").map_err(|err| err2no(&err))? {
+        "/sys/fs/selinux/enforce"
+    } else if exists("/selinux").map_err(|err| err2no(&err))? {
+        "/selinux/enforce"
+    } else {
+        return Ok(false);
+    };
+    let data: u8 = read_to_string(path)
+        .map_err(|err| err2no(&err))?
+        .parse()
+        .or(Err(Errno::EINVAL))?;
+    Ok(data != 0)
+}
+
 /// Seccomp sandbox profile export modes.
 #[derive(Debug, Clone, Copy, Eq, PartialEq)]
 pub enum ExportMode {
