@@ -71,7 +71,6 @@ use crate::{
     },
     err::err2no,
     hash::SydHashSet,
-    ioctl::{TIOCEXCL, TIOCGEXCL, TIOCGPTPEER, TIOCNXCL},
     path::{XPath, XPathBuf, PATH_MAX},
     proc::{proc_tgid, PROCMAP_QUERY},
     retry::retry_on_eintr,
@@ -165,17 +164,20 @@ pub fn set_pipemax<Fd: AsFd>(fd: Fd, size: c_int) -> Result<usize, Errno> {
 pub fn get_exclusive<Fd: AsFd>(fd: Fd) -> Result<bool, Errno> {
     let mut set: c_int = 0;
     let fd = fd.as_fd().as_raw_fd();
-    let req = TIOCGEXCL.ok_or(Errno::ENOTTY)?;
 
     // SAFETY: TIOCGEXCL takes an int* to return 0 or nonzero.
-    Errno::result(unsafe { syscall(SYS_ioctl, fd, req, std::ptr::addr_of_mut!(set)) })
+    Errno::result(unsafe { syscall(SYS_ioctl, fd, libc::TIOCGEXCL, std::ptr::addr_of_mut!(set)) })
         .map(|_| set != 0)
 }
 
 /// Set given terminal to exclusive mode, or disable exclusive mode.
 pub fn set_exclusive<Fd: AsFd>(fd: Fd, enable: bool) -> Result<(), Errno> {
     let fd = fd.as_fd().as_raw_fd();
-    let req = if enable { *TIOCEXCL } else { *TIOCNXCL }.ok_or(Errno::ENOTTY)?;
+    let req = if enable {
+        libc::TIOCEXCL
+    } else {
+        libc::TIOCNXCL
+    };
 
     // SAFETY: TIOC{E,N}XCL take no extra arguments.
     Errno::result(unsafe { syscall(SYS_ioctl, fd, req) }).map(drop)
@@ -187,11 +189,10 @@ pub fn set_exclusive<Fd: AsFd>(fd: Fd, enable: bool) -> Result<(), Errno> {
 pub fn openpts<Fd: AsFd>(fd: Fd, flags: OFlag) -> Result<OwnedFd, Errno> {
     let fd = fd.as_fd().as_raw_fd();
     let flags = flags.bits();
-    let req = TIOCGPTPEER.ok_or(Errno::ENOTTY)?;
 
     // SAFETY: In libc we trust.
     #[expect(clippy::cast_possible_truncation)]
-    Errno::result(unsafe { syscall(SYS_ioctl, fd, req, flags) }).map(|fd| {
+    Errno::result(unsafe { syscall(SYS_ioctl, fd, libc::TIOCGPTPEER, flags) }).map(|fd| {
         // SAFETY: TIOCGPTPEER returns a valid fd on success.
         unsafe { OwnedFd::from_raw_fd(fd as RawFd) }
     })
