@@ -30,7 +30,7 @@ use crate::{
     },
     config::HOOK_SCKCALLS,
     confine::{op2errno, op2name, scmp_arch_bits},
-    fs::{fd_status_flags, file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
+    fs::fd_status_flags,
     kernel::net::{
         accept::handle_accept,
         bind::handle_bind,
@@ -44,6 +44,7 @@ use crate::{
         sendto::handle_sendto,
         socket::{handle_socket, handle_socketpair},
     },
+    lookup::{file_type, safe_canonicalize, CanonicalPath, FileType, FsFlags},
     path::{XPath, XPathBuf},
     req::UNotifyEventRequest,
     sandbox::{Action, Capability, SandboxGuard},
@@ -85,13 +86,12 @@ pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
     let mut args = [0u64; ARGLEN];
     #[expect(clippy::arithmetic_side_effects)]
     let bufsiz = sizeof_ulong * ARGLEN;
-    let mut buf = Vec::new();
-    if buf.try_reserve(bufsiz).is_err() {
-        return request.fail_syscall(Errno::ENOMEM);
-    }
-    buf.resize(bufsiz, 0);
-    match request.read_mem(&mut buf, req.data.args[1]) {
-        Ok(n) if n == bufsiz => {
+    match request.read_vec(req.data.args[1], bufsiz) {
+        Ok(buf) if buf.len() != bufsiz => {
+            // Short read.
+            return request.fail_syscall(Errno::EFAULT);
+        }
+        Ok(buf) => {
             for (i, chunk) in buf.chunks_exact(sizeof_ulong).enumerate() {
                 match sizeof_ulong {
                     4 => match chunk.try_into() {
@@ -110,10 +110,7 @@ pub(crate) fn sys_socketcall(request: UNotifyEventRequest) -> ScmpNotifResp {
                 }
             }
         }
-        _ => {
-            // Short read or error.
-            return request.fail_syscall(Errno::EFAULT);
-        }
+        Err(errno) => return request.fail_syscall(errno),
     }
 
     syscall_network_handler(request, &args, op)
@@ -393,14 +390,10 @@ fn get_addr(
         return Err(Errno::EINVAL);
     }
     #[expect(clippy::arithmetic_side_effects)]
-    let addr_len = addr_len.min(std::mem::size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
+    let addr_len = addr_len.min(size_of::<libc::sockaddr_un>() + UNIX_PATH_MAX);
 
-    let mut addr = Vec::new();
-    addr.try_reserve(addr_len).or(Err(Errno::ENOMEM))?;
-    addr.resize(addr_len, 0);
-    let addr_len = addr_len.try_into().or(Err(Errno::EINVAL))?;
-
-    request.read_mem(&mut addr, addr_remote)?;
+    let addr = request.read_vec(addr_remote, addr_len)?;
+    let addr_len = addr.len().try_into().or(Err(Errno::EINVAL))?;
     let addr = addr.as_ptr().cast();
 
     // SAFETY: Invoking `SockaddrStorage::from_raw` is safe because:
@@ -410,13 +403,13 @@ fn get_addr(
     match unsafe { SockaddrStorage::from_raw(addr, Some(addr_len)) } {
         Some(addr)
             if addr.as_sockaddr_in().is_some()
-                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in>() =>
+                && (addr_len as usize) < size_of::<libc::sockaddr_in>() =>
         {
             Err(Errno::EINVAL)
         }
         Some(addr)
             if addr.as_sockaddr_in6().is_some()
-                && (addr_len as usize) < std::mem::size_of::<libc::sockaddr_in6>() =>
+                && (addr_len as usize) < size_of::<libc::sockaddr_in6>() =>
         {
             Err(Errno::EINVAL)
         }
@@ -517,7 +510,7 @@ fn canon_addr<'a>(
         // the sun_path field plus the length of the path (including the
         // null terminator).
         #[expect(clippy::arithmetic_side_effects)]
-        let size = std::mem::size_of::<libc::sa_family_t>() + socklen;
+        let size = size_of::<libc::sa_family_t>() + socklen;
 
         // SAFETY: We are converting a sockaddr_un to a
         // SockaddrStorage using a raw pointer. The sockaddr_un
@@ -640,28 +633,12 @@ pub(crate) fn sandbox_addr_unix(
         }
     };
 
-    // Convert /proc/${pid} to /proc/self as necessary.
-    let path = if let Some(p) = path.split_prefix(b"/proc") {
-        let mut buf = itoa::Buffer::new();
-        let req = request.scmpreq;
-        let pid = buf.format(req.pid);
-        if let Some(p) = p.split_prefix(pid.as_bytes()) {
-            let mut pdir = XPathBuf::from("/proc/self");
-            pdir.push(p.as_bytes());
-            Cow::Owned(pdir)
-        } else {
-            path
-        }
-    } else {
-        path
-    };
-
     // Check for access.
     let (action, filter) = sandbox.check_unix(caps, &path);
 
     if !filter {
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &path, "abs": abs,
                 "tip": format!("configure `allow/{caps}+{path}'"),
@@ -736,7 +713,7 @@ pub(crate) fn sandbox_addr_inet(
         // accept{,4}
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "block", "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
                 "tip": format!("configure `block-{addr}'"),
@@ -751,7 +728,7 @@ pub(crate) fn sandbox_addr_inet(
         // connect, sendto, send{m,}msg
         let ipv = if addr.is_ipv6() { 6 } else { 4 };
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "addr": format!("{addr}!{port}"), "ipv": ipv,
                 "tip": format!("configure `allow/{caps}+{addr}!{port}'"),
@@ -813,7 +790,7 @@ pub(crate) fn sandbox_addr_unnamed(
 
     if !filter {
         let sys = op2name(op);
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             warn!("ctx": "access", "cap": caps, "act": action,
                 "sys": sys, "unix": &name, "abs": true,
                 "tip": format!("configure `allow/{caps}+{name}'"),
