@@ -13,25 +13,25 @@ use std::{
 use libseccomp::ScmpNotifResp;
 use nix::{
     errno::Errno,
-    sys::socket::{sendmsg, ControlMessage, MsgFlags, SockaddrStorage, UnixCredentials},
+    sys::socket::{sendmsg, ControlMessage, SockaddrStorage, UnixCredentials},
     unistd::Pid,
 };
 use zeroize::Zeroizing;
 
 use crate::{
     compat::{
-        addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, PF_ALG, PF_INET, PF_INET6,
-        PF_NETLINK, PF_UNIX,
+        addr_family, cmsg_len_32, cmsg_space_32, msghdr, msghdr32, MsgFlags, PF_ALG, PF_INET,
+        PF_INET6, PF_NETLINK, PF_UNIX, UIO_MAXIOV, UIO_MAXLEN,
     },
     confine::scmp_arch_bits,
-    fs::{file_type, get_nonblock, has_recv_timeout, is_same_vm, FileType},
-    hook::UNotifyEventRequest,
+    fs::{get_nonblock, has_recv_timeout, is_same_vm},
     kernel::{
         net::{canon_addr, get_addr, sandbox_addr, to_msgflags},
         sandbox_path,
     },
     path::XPath,
-    sandbox::Capability,
+    req::UNotifyEventRequest,
+    sandbox::{Capability, Flags},
 };
 
 #[expect(clippy::cognitive_complexity)]
@@ -39,15 +39,13 @@ pub(crate) fn handle_sendmsg(
     fd: OwnedFd,
     request: &UNotifyEventRequest,
     args: &[u64; 6],
-    allow_unsupp_socket: bool,
-    restrict_oob: bool,
-    restrict_mkbdev: bool,
+    flags: Flags,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
-    let flags = to_msgflags(args[2])?;
+    let msgflags = to_msgflags(args[2])?;
 
     // SAFETY: Reject MSG_OOB as necessary.
-    if restrict_oob && flags.contains(MsgFlags::MSG_OOB) {
+    if !flags.allow_unsafe_oob() && msgflags.contains(MsgFlags::MSG_OOB) {
         // Signal no support to let the sandbox process
         // handle the error gracefully. This is consistent
         // with the Linux kernel.
@@ -63,10 +61,7 @@ pub(crate) fn handle_sendmsg(
     } else {
         size_of::<msghdr>()
     };
-    let mut buf = Zeroizing::new(Vec::new());
-    buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-    buf.resize(size, 0);
-    request.read_mem(&mut buf, args[1])?;
+    let buf = request.read_vec_zeroed(args[1], size)?;
 
     let msg = if is32 {
         // SAFETY: See below.
@@ -90,13 +85,10 @@ pub(crate) fn handle_sendmsg(
 
     // Step 1: Handle the address.
     let sandbox = request.get_sandbox();
+    let log_scmp = sandbox.log_scmp();
     let addr = if !addr_remote.is_null() && addr_len > 0 {
-        let (addr, root) = canon_addr(
-            request,
-            &sandbox,
-            get_addr(request, addr_remote as u64, addr_len)?,
-            Capability::CAP_NET_CONNECT,
-        )?;
+        let addr = get_addr(request, addr_remote as u64, addr_len)?;
+        let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
         match addr_family(&addr) {
             PF_UNIX | PF_INET | PF_INET6 => {
                 // Check for access.
@@ -112,9 +104,8 @@ pub(crate) fn handle_sendmsg(
             PF_ALG | PF_NETLINK => {
                 // SAFETY: We do not check AF_ALG or AF_NETLINK for access.
             }
-            _ if allow_unsupp_socket => {
-                // SAFETY: No pointer-dereference in access check.
-                return unsafe { Ok(request.continue_syscall()) };
+            _ if flags.allow_unsupp_socket() => {
+                // SAFETY: We do not check unsupported sockets for access.
             }
             _ => return Err(Errno::EAFNOSUPPORT),
         };
@@ -133,11 +124,8 @@ pub(crate) fn handle_sendmsg(
         #[expect(clippy::useless_conversion)]
         let cmsg_len = usize::try_from(msg.msg_controllen)
             .or(Err(Errno::EINVAL))?
-            .min(1000000); // SAFETY: Cap at 1mio.
-        let mut cmsg_buf = Vec::new();
-        cmsg_buf.try_reserve(cmsg_len).or(Err(Errno::ENOMEM))?;
-        cmsg_buf.resize(cmsg_len, 0);
-        request.read_mem(&mut cmsg_buf, msg.msg_control as u64)?;
+            .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+        let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
         Some(parse_control_messages(request, &cmsg_buf)?)
     } else {
         None
@@ -186,22 +174,9 @@ pub(crate) fn handle_sendmsg(
         }
 
         for fds in control_fds {
-            // SAFETY: Deny sending file descriptors referring to
-            // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
-            // 2. Directories
-            // 3. Symbolic links
-            //
-            // Note, we do allow files of unknown type such as epoll
-            // fds and event fds as some programs such as pipewire
-            // depend on this. See test-pw-filter test of pipewire
-            // for more information about this.
+            // SAFETY: Check safe file types for SCM_RIGHTS.
             for fd in fds {
-                // TODO: Log this deny!
-                match file_type(fd, None, false)? {
-                    FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
-                    FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
-                    _ => {}
-                }
+                request.check_scm_rights(fd, flags, 0x10, log_scmp)?;
             }
 
             // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -265,28 +240,20 @@ pub(crate) fn handle_sendmsg(
         // SAFETY: The msg_iovlen member of the msghdr struct
         // must not be fully trusted, it can be overly large,
         // and allocating a Vector of that capacity may overflow.
+        // Here we cap it to UIO_MAXIOV for safety.
         #[expect(clippy::useless_conversion)]
         let len = usize::try_from(msg.msg_iovlen)
-            .or(Err(Errno::EINVAL))?
-            .min(1000000); // Cap count at 1mio
+            .or(Err(Errno::EMSGSIZE))?
+            .min(UIO_MAXIOV);
+
         let size = if is32 {
             len.checked_mul(size_of::<crate::compat::iovec32>())
         } else {
             len.checked_mul(size_of::<libc::iovec>())
         }
-        .ok_or(Errno::EINVAL)?;
-        let mut buf = Zeroizing::new(Vec::new());
-        buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-        buf.resize(size, 0);
-        request.read_mem(&mut buf, msg.msg_iov as u64)?;
+        .ok_or(Errno::EMSGSIZE)?;
 
-        // SAFETY: This operation assumes that the buffer (`buf`) contains a valid sequence of bytes
-        // that correctly represent an array of `iovec` structures. This is ensured by the preceding
-        // code that reads memory into `buf` with proper length calculation. The length `len` is
-        // derived from `msg.msg_iovlen` and capped to prevent overflow, ensuring that we do not
-        // exceed the allocation size of `buf`. The conversion to a pointer and then to a slice
-        // of `iovec` is safe under these conditions, assuming the memory layout of `iovec` is
-        // correct and `buf` is correctly sized and aligned.
+        let buf = request.read_vec_zeroed(msg.msg_iov as u64, size)?;
         let mut iovecs: Vec<libc::iovec> = Vec::new();
         if is32 {
             for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
@@ -304,25 +271,19 @@ pub(crate) fn handle_sendmsg(
 
         for iov in iovecs {
             if iov.iov_base.is_null() || iov.iov_len == 0 {
-                // XXX: This happens with socketcall on x86, why?
                 continue;
             }
 
-            // Cap the length to a maximum value to avoid large allocations.
-            // SAFETY: The maximum length cap prevents excessive memory
-            // allocation based on untrusted `iov_len`.
-            let iov_len = iov.iov_len.min(1000000); // Cap count at 1mio
-
-            // Allocate a buffer to read into. This buffer size is now capped.
-            let mut data_buf = Vec::new();
-            data_buf.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
-            data_buf.resize(iov_len, 0);
+            // Validate length to avoid large allocations.
+            #[expect(clippy::useless_conversion)]
+            let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
+            if iov_len > UIO_MAXLEN {
+                // Invalid iovec count: Too large.
+                return Err(Errno::EMSGSIZE);
+            }
 
             // Read the memory from the remote process into our buffer.
-            // SAFETY: This operation relies on the correctness of
-            // `iov_base` as a pointer into the remote process's memory and
-            // the capped `iov_len`.
-            request.read_mem(&mut data_buf, iov.iov_base as u64)?;
+            let data_buf = request.read_vec(iov.iov_base as u64, iov_len)?;
 
             // Keep the pointer accessible, IoSlice needs a valid reference.
             io_buffers.try_reserve(1).or(Err(Errno::ENOMEM))?;
@@ -337,7 +298,7 @@ pub(crate) fn handle_sendmsg(
     }
 
     // SAFETY: Record blocking call so it can get invalidated.
-    let is_blocking = if !flags.contains(MsgFlags::MSG_DONTWAIT) && !get_nonblock(&fd)? {
+    let is_blocking = if !msgflags.contains(MsgFlags::MSG_DONTWAIT) && !get_nonblock(&fd)? {
         let req = request.scmpreq;
         let ignore_restart = has_recv_timeout(&fd)?;
 
@@ -349,22 +310,25 @@ pub(crate) fn handle_sendmsg(
         false
     };
 
-    #[expect(clippy::cast_possible_wrap)]
     let result = if let Some((addr, _)) = addr {
         // UNIX domain/abstract socket.
         sendmsg(
             fd.as_raw_fd(),
             &io_slices,
             &control_messages,
-            flags,
+            msgflags.into(),
             Some(&addr),
         )
     } else {
         // Connection-mode socket.
-        sendmsg::<SockaddrStorage>(fd.as_raw_fd(), &io_slices, &control_messages, flags, None)
-    }
-    .map(|n| request.return_syscall(n as i64));
-    drop(fd); // Close our copy of the socket.
+        sendmsg::<SockaddrStorage>(
+            fd.as_raw_fd(),
+            &io_slices,
+            &control_messages,
+            msgflags.into(),
+            None,
+        )
+    };
 
     // Remove invalidation record unless interrupted.
     if is_blocking {
@@ -373,7 +337,16 @@ pub(crate) fn handle_sendmsg(
             .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
     }
 
-    result
+    // Send SIGPIPE for EPIPE unless MSG_NOSIGNAL is set.
+    #[expect(clippy::cast_possible_wrap)]
+    Ok(match result {
+        Ok(n) => request.return_syscall(n as i64),
+        Err(Errno::EPIPE) if !msgflags.contains(MsgFlags::MSG_NOSIGNAL) => {
+            request.pidfd_kill(libc::SIGPIPE)?;
+            request.fail_syscall(Errno::EPIPE)
+        }
+        Err(errno) => request.fail_syscall(errno),
+    })
 }
 
 #[expect(clippy::cognitive_complexity)]
@@ -381,15 +354,13 @@ pub(crate) fn handle_sendmmsg(
     fd: OwnedFd,
     request: &UNotifyEventRequest,
     args: &[u64; 6],
-    allow_unsupp_socket: bool,
-    restrict_oob: bool,
-    restrict_mkbdev: bool,
+    flags: Flags,
 ) -> Result<ScmpNotifResp, Errno> {
     // SAFETY: Reject undefined/invalid flags.
     let msgflags = to_msgflags(args[3])?;
 
     // SAFETY: Reject MSG_OOB as necessary.
-    if restrict_oob && msgflags.contains(MsgFlags::MSG_OOB) {
+    if !flags.allow_unsafe_oob() && msgflags.contains(MsgFlags::MSG_OOB) {
         // Signal no support to let the sandbox process
         // handle the error gracefully. This is consistent
         // with the Linux kernel.
@@ -414,18 +385,16 @@ pub(crate) fn handle_sendmmsg(
     }
     .ok_or(Errno::EINVAL)?;
 
-    // Read mmsghdr structures from remote process memory
-    let mut buf = Zeroizing::new(Vec::new());
-    buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-    buf.resize(size, 0);
-    request.read_mem(&mut buf, addr)?;
+    // Read mmsghdr structures from remote process memory.
+    let buf = request.read_vec_zeroed(addr, size)?;
 
     let mut mmsghdrs: Vec<crate::compat::mmsghdr> = Vec::new();
     if is32 {
         for chunk in buf.chunks(size_of::<crate::compat::mmsghdr32>()) {
             // SAFETY: See the relevant comment in handle_sendmsg.
-            let mmsghdr: crate::compat::mmsghdr32 =
-                unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) };
+            let mmsghdr: crate::compat::mmsghdr32 = unsafe {
+                std::ptr::read_unaligned(chunk.as_ptr() as *const crate::compat::mmsghdr32)
+            };
             mmsghdrs.try_reserve(1).or(Err(Errno::ENOMEM))?;
             mmsghdrs.push(mmsghdr.into());
         }
@@ -433,7 +402,9 @@ pub(crate) fn handle_sendmmsg(
         for chunk in buf.chunks(size_of::<crate::compat::mmsghdr>()) {
             mmsghdrs.try_reserve(1).or(Err(Errno::ENOMEM))?;
             // SAFETY: See the relevant comment in handle_sendmsg.
-            mmsghdrs.push(unsafe { std::ptr::read_unaligned(chunk.as_ptr() as *const _) });
+            mmsghdrs.push(unsafe {
+                std::ptr::read_unaligned(chunk.as_ptr() as *const crate::compat::mmsghdr)
+            });
         }
     };
 
@@ -444,6 +415,8 @@ pub(crate) fn handle_sendmmsg(
         } else {
             (false, false)
         };
+    // Check if we want to send SIGPIPE on EPIPE.
+    let must_signal = !msgflags.contains(MsgFlags::MSG_NOSIGNAL);
 
     // Prepare a series of sendmsg calls.
     for mmsg in &mut mmsghdrs {
@@ -453,13 +426,10 @@ pub(crate) fn handle_sendmmsg(
         let addr_remote = msg.msg_name;
         let addr_len = msg.msg_namelen;
         let sandbox = request.get_sandbox();
+        let log_scmp = sandbox.log_scmp();
         let addr_root = if !addr_remote.is_null() && addr_len > 0 {
-            let (addr, root) = canon_addr(
-                request,
-                &sandbox,
-                get_addr(request, addr_remote as u64, addr_len)?,
-                Capability::CAP_NET_CONNECT,
-            )?;
+            let addr = get_addr(request, addr_remote as u64, addr_len)?;
+            let (addr, root) = canon_addr(request, &sandbox, &addr, Capability::CAP_NET_CONNECT)?;
             match addr_family(&addr) {
                 PF_UNIX | PF_INET | PF_INET6 => {
                     // Check for access.
@@ -475,9 +445,8 @@ pub(crate) fn handle_sendmmsg(
                 PF_ALG | PF_NETLINK => {
                     // SAFETY: We do not check AF_ALG and AF_NETLINK for access.
                 }
-                _ if allow_unsupp_socket => {
-                    // SAFETY: No pointer-dereference in access check.
-                    return unsafe { Ok(request.continue_syscall()) };
+                _ if flags.allow_unsupp_socket() => {
+                    // SAFETY: We do not check unsupported sockets for access.
                 }
                 _ => return Err(Errno::EAFNOSUPPORT),
             };
@@ -496,11 +465,8 @@ pub(crate) fn handle_sendmmsg(
             #[expect(clippy::useless_conversion)]
             let cmsg_len = usize::try_from(msg.msg_controllen)
                 .or(Err(Errno::EINVAL))?
-                .min(1000000); // SAFETY: Cap at 1mio.
-            let mut cmsg_buf = Vec::new();
-            cmsg_buf.try_reserve(cmsg_len).or(Err(Errno::ENOMEM))?;
-            cmsg_buf.resize(cmsg_len, 0);
-            request.read_mem(&mut cmsg_buf, msg.msg_control as u64)?;
+                .min(UIO_MAXLEN); // SAFETY: Cap at UIO_MAXLEN.
+            let cmsg_buf = request.read_vec(msg.msg_control as u64, cmsg_len)?;
             Some(parse_control_messages(request, &cmsg_buf)?)
         } else {
             None
@@ -533,7 +499,7 @@ pub(crate) fn handle_sendmmsg(
                             &sandbox,
                             addr,
                             root,
-                            0x10,
+                            0x14,
                             Capability::CAP_NET_SENDFD,
                         )?;
                     } else {
@@ -553,22 +519,9 @@ pub(crate) fn handle_sendmmsg(
                 }
 
                 for fds in control_fds {
-                    // SAFETY: Deny sending file descriptors referring to
-                    // 1. Block devices unless trace/allow_unsafe_mkbdev:1 is set.
-                    // 2. Directories
-                    // 3. Symbolic links
-                    //
-                    // Note, we do allow files of unknown type such as epoll
-                    // fds and event fds as some programs such as pipewire
-                    // depend on this. See test-pw-filter test of pipewire
-                    // for more information about this.
+                    // SAFETY: Check safe file types for SCM_RIGHTS.
                     for fd in fds {
-                        // TODO: Log this deny!
-                        match file_type(fd, None, false)? {
-                            FileType::Dir | FileType::Lnk => return Err(Errno::EACCES),
-                            FileType::Blk if restrict_mkbdev => return Err(Errno::EACCES),
-                            _ => {}
-                        }
+                        request.check_scm_rights(fd, flags, 0x14, log_scmp)?;
                     }
 
                     // SAFETY: OwnedFd is repr(transparent) over RawFd, so
@@ -634,21 +587,20 @@ pub(crate) fn handle_sendmmsg(
             // SAFETY: The msg_iovlen member of the msghdr struct
             // must not be fully trusted, it can be overly large,
             // and allocating a Vector of that capacity may overflow.
+            // Here we cap it to UIO_MAXIOV for safety.
             #[expect(clippy::useless_conversion)]
             let len = usize::try_from(msg.msg_iovlen)
-                .or(Err(Errno::EINVAL))?
-                .min(1000000); // Cap count at 1mio.
+                .or(Err(Errno::EMSGSIZE))?
+                .min(UIO_MAXIOV);
+
             let size = if is32 {
                 len.checked_mul(size_of::<crate::compat::iovec32>())
             } else {
                 len.checked_mul(size_of::<libc::iovec>())
             }
-            .ok_or(Errno::EINVAL)?;
-            let mut buf = Zeroizing::new(Vec::new());
-            buf.try_reserve(size).or(Err(Errno::ENOMEM))?;
-            buf.resize(size, 0);
-            request.read_mem(&mut buf, msg.msg_iov as u64)?;
+            .ok_or(Errno::EMSGSIZE)?;
 
+            let buf = request.read_vec_zeroed(msg.msg_iov as u64, size)?;
             let mut iovecs: Vec<libc::iovec> = Vec::new();
             if is32 {
                 for chunk in buf.chunks(size_of::<crate::compat::iovec32>()) {
@@ -667,23 +619,23 @@ pub(crate) fn handle_sendmmsg(
             };
 
             for iov in iovecs {
-                // Cap the length to a maximum value to avoid large allocations.
-                // SAFETY: The maximum length cap prevents excessive memory
-                // allocation based on untrusted `iov_len`.
-                let iov_len = iov.iov_len.min(1000000); // Cap count at 1mio
+                if iov.iov_base.is_null() || iov.iov_len == 0 {
+                    continue;
+                }
 
-                // Allocate a buffer to read into. This buffer size is now capped.
-                let mut data_buf = Vec::new();
-                data_buf.try_reserve(iov_len).or(Err(Errno::ENOMEM))?;
-                data_buf.resize(iov_len, 0);
+                // Validate length to avoid large allocations.
+                #[expect(clippy::useless_conversion)]
+                let iov_len = usize::try_from(iov.iov_len).or(Err(Errno::EMSGSIZE))?;
+                if iov_len > UIO_MAXLEN {
+                    // Invalid iovec count: Too large.
+                    return Err(Errno::EMSGSIZE);
+                }
 
                 // Read the memory from the remote process into our buffer.
-                // SAFETY: This operation relies on the correctness of
-                // `iov_base` as a pointer into the remote process's memory and
-                // the capped `iov_len`.
-                request.read_mem(&mut data_buf, iov.iov_base as u64)?;
+                let data_buf = request.read_vec(iov.iov_base as u64, iov_len)?;
 
                 // Keep the pointer accessible, IoSlice needs a valid reference.
+                io_buffers.try_reserve(1).or(Err(Errno::ENOMEM))?;
                 io_buffers.push(data_buf);
             }
             for buffer in &io_buffers {
@@ -703,7 +655,7 @@ pub(crate) fn handle_sendmmsg(
                     fd.as_raw_fd(),
                     &io_slices,
                     &control_messages,
-                    msgflags,
+                    msgflags.into(),
                     Some(&addr),
                 )
             } else {
@@ -712,7 +664,7 @@ pub(crate) fn handle_sendmmsg(
                     fd.as_raw_fd(),
                     &io_slices,
                     &control_messages,
-                    msgflags,
+                    msgflags.into(),
                     None,
                 )
             };
@@ -724,10 +676,19 @@ pub(crate) fn handle_sendmmsg(
                     .del_sys_block(req.id, matches!(result, Err(Errno::EINTR)))?;
             }
 
+            // Send SIGPIPE for EPIPE unless MSG_NOSIGNAL is set.
+            mmsg.msg_len = match result {
+                Ok(n) => n.try_into().or(Err(Errno::EINVAL))?,
+                Err(Errno::EPIPE) if must_signal => {
+                    request.pidfd_kill(libc::SIGPIPE)?;
+                    return Err(Errno::EPIPE);
+                }
+                Err(errno) => return Err(errno),
+            };
+
             mmsg.msg_len = result?.try_into().or(Err(Errno::EINVAL))?;
         }
     }
-    drop(fd); // Close our copy of the socket.
 
     // Write back mmsghdr structures to remote process memory
     let mut buf: Zeroizing<Vec<u8>> = Zeroizing::new(Vec::new());
@@ -798,10 +759,10 @@ fn parse_control_messages(
     while offset < cmsg_buf.len() {
         // SAFETY: Ensuring alignment for `cmsghdr` by starting from a u8 pointer.  The
         // `cmsg_buf` is originally a u8 buffer, which may not satisfy the alignment
-        // requirements of `cmsghdr`.  This cast assumes that the buffer provided by
+        // requirements of `cmsghdr`. This cast assumes that the buffer provided by
         // `request.process.read_mem` is correctly aligned for `cmsghdr` structures, which is true if
         // the buffer is initially populated in a manner adhering to the alignment
-        // requirements of `cmsghdr`.  The caller is responsible for ensuring that `offset`
+        // requirements of `cmsghdr`. The caller is responsible for ensuring that `offset`
         // is correctly aligned for `cmsghdr` when accessing the buffer.
         let (cmsg_header, cmsg_len0): (crate::compat::cmsghdr, usize) = if is32 {
             // SAFETY: See the comment above.
@@ -1060,16 +1021,13 @@ fn parse_control_messages(
                 control_tx_times.push(tx_time); // Store the Tx time
             }
             (libc::IPPROTO_IP, libc::IP_TOS) => {
-                // IP_TOS ancillary data is an int.
-                if data_len != size_of::<libc::c_int>() {
+                // IP_TOS ancillary data is a u8.
+                if data_len != size_of::<u8>() {
                     return Err(Errno::EINVAL);
                 }
-                // SAFETY: Exact int-sized payload validated; bounds and alignment guaranteed
-                // by prior checks and cmsg parsing semantics. Single c_int read is safe.
-                #[expect(clippy::cast_ptr_alignment)]
-                let val = unsafe { *(data.as_ptr() as *const libc::c_int) };
-                // See ip_cmd_send() in Linux' net/ipv4/ip_sockglue.c.
-                let val: u8 = val.try_into().or(Err(Errno::EINVAL))?;
+                // SAFETY: Exact u8-sized payload validated; bounds and alignment guaranteed
+                // by prior checks and cmsg parsing semantics. Single u8 read is safe.
+                let val = unsafe { *data.as_ptr() };
                 control_ipv4_toses.push(val);
             }
             (libc::IPPROTO_IPV6, libc::IPV6_TCLASS) => {
