@@ -23,7 +23,7 @@ use nix::{
     errno::Errno,
     fcntl::{OFlag, OpenHow, ResolveFlag},
     sys::stat::{umask, Mode},
-    unistd::Uid,
+    unistd::{Pid, Uid},
     NixPath,
 };
 
@@ -259,7 +259,7 @@ fn syscall_open_handler(
             return Err(Errno::EACCES);
         }
 
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             crate::debug!("ctx": "open", "op": "init",
                 "cap": caps,
                 "sbc": *sandbox.state,
@@ -294,7 +294,7 @@ fn syscall_open_handler(
         arg.flags.insert(SysFlags::CHECK_MAGIC);
         let (mut path, magic, _) = request.read_path(&sandbox, arg)?;
 
-        if sandbox.verbose {
+        if sandbox.log_scmp() {
             crate::debug!("ctx": "open", "op": "read_path",
                 "path": &path, "magic": magic,
                 "req": &request);
@@ -502,26 +502,10 @@ fn syscall_open_handler(
         let mut filter = false;
 
         let mut path_check = if !caps.is_empty() {
-            // Convert /proc/${pid} to /proc/self as necessary.
-            let path_check = if let Some(p) = path.abs().split_prefix(b"/proc") {
-                let mut buf = itoa::Buffer::new();
-                let req = request.scmpreq;
-                let pid = buf.format(req.pid);
-                if let Some(p) = p.split_prefix(pid.as_bytes()) {
-                    let mut pdir = XPathBuf::from("/proc/self");
-                    pdir.push(p.as_bytes());
-                    Cow::Owned(pdir)
-                } else {
-                    Cow::Borrowed(path.abs())
-                }
-            } else {
-                Cow::Borrowed(path.abs())
-            };
-
             // If open is a TTY both the virtual and real path are checked for access.
             // The more critical action wins.
             let mut flip_tty = false;
-            for (idx, path) in [Some(&path_check), path_tty.as_ref()]
+            for (idx, path) in [Some(path.abs()), path_tty.as_deref()]
                 .iter()
                 .flatten()
                 .enumerate()
@@ -541,7 +525,7 @@ fn syscall_open_handler(
             if flip_tty {
                 path_tty.unwrap()
             } else {
-                path_check
+                Cow::Borrowed(path.abs())
             }
         } else {
             Cow::Borrowed(path.abs())
@@ -590,7 +574,7 @@ fn syscall_open_handler(
                 flags.insert(OFlag::O_APPEND);
                 flags.remove(OFlag::O_TRUNC);
 
-                if sandbox.verbose {
+                if sandbox.log_scmp() {
                     info!("ctx": "open", "op": "set_append_only",
                             "msg": "added O_APPEND and removed O_TRUNC from open flags",
                             "sys": request.syscall, "path": &path_check,
@@ -611,7 +595,7 @@ fn syscall_open_handler(
         };
 
         let force_umask = sandbox.umask;
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
         let unsafe_open_kfd = sandbox.flags.allow_unsafe_open_kfd();
         let unsafe_open_path = sandbox.flags.allow_unsafe_open_path();
         let filter_machine_id = !sandbox.flags.allow_unsafe_machine_id();
@@ -628,7 +612,7 @@ fn syscall_open_handler(
         // Perform action: allow->emulate, deny->log.
         if !filter && action >= Action::Warn && log_enabled!(LogLevel::Warn) {
             let grp = caps.to_string().to_ascii_lowercase();
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "access", "cap": caps, "act": action,
                     "sys": request.syscall, "path": &path_check,
                     "mode": mode.bits(),
@@ -733,7 +717,7 @@ fn syscall_open_handler(
             }
 
             if let Some(msg) = err {
-                if verbose {
+                if log_scmp {
                     error!("ctx": "trusted_file_creation", "cap": caps,
                         "sys": request.syscall, "path": &path_check,
                         "mode": mode, "oflags": format_oflags(flags), "rflags": arg.fsflags,
@@ -876,7 +860,7 @@ fn syscall_open_handler(
                         // Existing path: Construct path to proc(5) magic symlink,
                         // or pre-open file descriptor.
                         let fd = do_open(&request, &path, flags, is_blocking)?;
-                        do_open_post(fd, path, filter_proc_status)?
+                        do_open_post(fd, request.scmpreq.pid(), path, filter_proc_status)?
                     }
                 } else if o_mask {
                     // Cannot be O_PATH or encrypted fd!
@@ -1246,12 +1230,13 @@ fn do_open_pre(
 
 fn do_open_post(
     mut fd: OwnedFd,
+    pid: Pid,
     path: CanonicalPath,
     filter_proc_status: bool,
 ) -> Result<OwnedFd, Errno> {
     // SAFETY: Filter security-sensitive proc_pid_status(5) contents.
-    if filter_proc_status && path.abs().is_proc_pid_status() {
-        fd = do_mask_proc_pid_status(fd, path)?;
+    if filter_proc_status && path.abs().is_proc_status() {
+        fd = do_mask_proc_pid_status(fd, pid, path)?;
     }
 
     Ok(fd)
@@ -1330,9 +1315,15 @@ fn do_open_proc_osrelease() -> Result<OwnedFd, Errno> {
     Ok(mfd.into())
 }
 
-fn do_mask_proc_pid_status(fd: OwnedFd, path: CanonicalPath) -> Result<OwnedFd, Errno> {
+fn do_mask_proc_pid_status(fd: OwnedFd, pid: Pid, path: CanonicalPath) -> Result<OwnedFd, Errno> {
     // Prepare the name of the memfd for identification.
+    // Undo /proc/self -> /proc/$pid.
     let mut path = path.take();
+    if path.starts_with(b"/proc/self/") {
+        let mut pfd = XPathBuf::from("/proc");
+        pfd.push_pid(pid);
+        path.replace_prefix(b"/proc/self", pfd.as_bytes())?;
+    }
     path.replace_prefix(b"/", b"syd/")?;
 
     // Open a memory fd to be sealed for write.
