@@ -39,11 +39,11 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let sandbox = request.get_sandbox();
         let allowed = sandbox.chk_uid_transit(source_uid, target_uid);
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
         drop(sandbox); // release the read lock.
 
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_uid": target_uid.as_raw(), "source_uid": source_uid.as_raw(),
@@ -63,7 +63,7 @@ pub(crate) fn sys_setuid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         // SAFETY: nix version of setuid does not allow -1 as argument.
         if let Err(errno) = Errno::result(unsafe { libc::setuid(target_uid.as_raw()) }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_uid": target_uid.as_raw(), "source_uid": source_uid.as_raw(),
@@ -112,11 +112,11 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         let sandbox = request.get_sandbox();
         let allowed = sandbox.chk_gid_transit(source_gid, target_gid);
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
         drop(sandbox); // release the read lock.
 
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_gid": target_gid.as_raw(), "source_gid": source_gid.as_raw(),
@@ -136,7 +136,7 @@ pub(crate) fn sys_setgid(request: UNotifyEventRequest) -> ScmpNotifResp {
 
         // SAFETY: nix version of setgid does not allow -1 as argument.
         if let Err(errno) = Errno::result(unsafe { libc::setgid(target_gid.as_raw()) }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_gid": target_gid.as_raw(), "source_gid": source_gid.as_raw(),
@@ -224,13 +224,13 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support RUID != EUID
         if let Some(target_ruid) = target_ruid {
             if let Some(target_euid) = target_euid {
                 if target_ruid != target_euid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_euid": target_euid.as_raw(), "target_ruid": target_ruid.as_raw(),
@@ -270,7 +270,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_ruid = target_ruid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         let target_euid = target_euid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_euid": target_euid, "target_ruid": target_ruid,
@@ -296,7 +296,7 @@ pub(crate) fn sys_setreuid(request: UNotifyEventRequest) -> ScmpNotifResp {
                 libc::syscall(libc::SYS_setreuid, target_ruid, target_euid)
             })
         {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_euid": target_euid, "target_ruid": target_ruid,
@@ -386,13 +386,13 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support Rgid != Egid
         if let Some(target_rgid) = target_rgid {
             if let Some(target_egid) = target_egid {
                 if target_rgid != target_egid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_egid": target_egid.as_raw(), "target_rgid": target_rgid.as_raw(),
@@ -432,7 +432,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_rgid = target_rgid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         let target_egid = target_egid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_egid": target_egid, "target_rgid": target_rgid,
@@ -458,7 +458,7 @@ pub(crate) fn sys_setregid(request: UNotifyEventRequest) -> ScmpNotifResp {
                 libc::syscall(libc::SYS_setregid, target_rgid, target_egid)
             })
         {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_egid": target_egid, "target_rgid": target_rgid,
@@ -566,13 +566,13 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support RUID != EUID != SUID
         if let Some(target_ruid) = target_ruid {
             if let Some(target_euid) = target_euid {
                 if target_ruid != target_euid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.map(|u| u.as_raw()),
@@ -604,7 +604,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_ruid) = target_ruid {
             if let Some(target_suid) = target_suid {
                 if target_ruid != target_suid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.as_raw(),
@@ -636,7 +636,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_euid) = target_euid {
             if let Some(target_suid) = target_suid {
                 if target_euid != target_suid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_suid": target_suid.as_raw(),
@@ -692,7 +692,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_euid = target_euid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         let target_suid = target_suid.map(|uid| i64::from(uid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_suid": target_suid,
@@ -724,7 +724,7 @@ pub(crate) fn sys_setresuid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Err(errno) = Errno::result(unsafe {
             libc::syscall(libc::SYS_setresuid, target_ruid, target_euid, target_suid)
         }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_suid": target_suid,
@@ -840,13 +840,13 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         }
 
         let sandbox = request.get_sandbox();
-        let verbose = sandbox.verbose;
+        let log_scmp = sandbox.log_scmp();
 
         // SAFETY: We do not support Rgid != Egid != Sgid
         if let Some(target_rgid) = target_rgid {
             if let Some(target_egid) = target_egid {
                 if target_rgid != target_egid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.map(|u| u.as_raw()),
@@ -878,7 +878,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_rgid) = target_rgid {
             if let Some(target_sgid) = target_sgid {
                 if target_rgid != target_sgid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.as_raw(),
@@ -910,7 +910,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Some(target_egid) = target_egid {
             if let Some(target_sgid) = target_sgid {
                 if target_egid != target_sgid {
-                    if verbose {
+                    if log_scmp {
                         warn!("ctx": "safesetid", "err": libc::EACCES,
                             "sys": request.syscall, "req": &request,
                             "target_sgid": target_sgid.as_raw(),
@@ -966,7 +966,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         let target_egid = target_egid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         let target_sgid = target_sgid.map(|gid| i64::from(gid.as_raw())).unwrap_or(-1);
         if !allowed {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": libc::EACCES,
                     "sys": request.syscall, "req": request,
                     "target_sgid": target_sgid,
@@ -998,7 +998,7 @@ pub(crate) fn sys_setresgid(request: UNotifyEventRequest) -> ScmpNotifResp {
         if let Err(errno) = Errno::result(unsafe {
             libc::syscall(libc::SYS_setresgid, target_rgid, target_egid, target_sgid)
         }) {
-            if verbose {
+            if log_scmp {
                 warn!("ctx": "safesetid", "err": errno as i32,
                     "sys": request.syscall, "req": request,
                     "target_sgid": target_sgid,
