@@ -242,8 +242,9 @@ use serde::{Serialize, Serializer};
 use crate::{
     compat::{fstatx, lsm_list_modules},
     confine::{
-        check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled, ns_enabled,
-        seccomp_arch_native_name, vdso_list_calls, SydPersona,
+        apparmor_enabled, check_cross_memory_attach, check_unix_diag, is_coredump, lock_enabled,
+        ns_enabled, seccomp_arch_native_name, selinux_enabled, selinux_enforced, vdso_list_calls,
+        SydPersona,
     },
     err::err2no,
     hash::{aes_ctr_info, check_setsockopt_serial_support, hmac_sha256_info, key_ring_validate},
@@ -261,10 +262,6 @@ use crate::{
 macro_rules! main {
     { $($body:tt)* } => {
         fn main() -> std::process::ExitCode {
-            // Silence mimalloc warnings.
-            #[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
-            syd::config_mimalloc();
-
             match (|| -> syd::err::SydResult<std::process::ExitCode> { $($body)* })() {
                 Ok(code) => code,
                 Err(err) => {
@@ -324,6 +321,19 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
         return Ok(());
     }
 
+    let alloc = if cfg!(all(
+        not(feature = "prof"),
+        target_page_size_4k,
+        target_pointer_width = "64"
+    )) {
+        "GrapheneOS"
+    } else if cfg!(feature = "prof") {
+        "TCMalloc"
+    } else {
+        "Libc"
+    };
+    printfln!("Allocator: {alloc}")?;
+
     let libapi = libseccomp::get_api();
     match ScmpVersion::current() {
         Ok(libver) => {
@@ -451,6 +461,27 @@ pub fn syd_info(verbose: bool) -> Result<(), Errno> {
     };
     printfln!("LSMs: {lsms}.")?;
 
+    // Log SELinux and Apparmor status.
+    let selinux = match selinux_enabled() {
+        Ok(true) => {
+            let enforce = if selinux_enforced().unwrap_or(false) {
+                "Enforcing"
+            } else {
+                "Permissive"
+            };
+            format!("enabled ({enforce})")
+        }
+        Ok(false) => "disabled".to_string(),
+        Err(errno) => format!("? (error: {errno})"),
+    };
+    let apparmor = match apparmor_enabled() {
+        Ok(true) => "enabled".to_string(),
+        Ok(false) => "disabled".to_string(),
+        Err(errno) => format!("? (error: {errno})"),
+    };
+    printfln!("SELinux is {selinux}.")?;
+    printfln!("AppArmor is {apparmor}.")?;
+
     // List vDSO calls.
     match vdso_list_calls() {
         Ok(names) if names.is_empty() => printfln!("No vDSO calls found.")?,
@@ -1166,40 +1197,6 @@ pub fn set_sigpipe_dfl() -> Result<(), Errno> {
     unsafe { signal(Signal::SIGPIPE, SigHandler::SigDfl) }.map(drop)
 }
 
-/// Configure mimalloc to silence stderr.
-///
-/// This function is NOT thread-safe, call it early at init.
-#[cfg(all(not(feature = "prof"), target_pointer_width = "64"))]
-#[allow(non_upper_case_globals)]
-pub fn config_mimalloc() {
-    // Extended feature is not usable (breaks on loongarch64),
-    // therefore we define a subset of mimalloc API here.
-    #[allow(non_camel_case_types)]
-    type mi_option_t = libc::c_int;
-    #[allow(non_camel_case_types)]
-    type mi_output_fun =
-        Option<unsafe extern "C" fn(msg: *const libc::c_char, arg: *mut libc::c_void)>;
-
-    #[link(name = "mimalloc")]
-    extern "C" {
-        fn mi_option_set_enabled(option: mi_option_t, enable: bool);
-        fn mi_register_output(out: mi_output_fun, arg: *mut libc::c_void);
-    }
-    extern "C" fn syd_noop_output(_msg: *const libc::c_char, _arg: *mut libc::c_void) {}
-
-    const mi_option_show_errors: mi_option_t = 0;
-    const mi_option_show_stats: mi_option_t = 1;
-    const mi_option_verbose: mi_option_t = 2;
-
-    // SAFETY: In mimalloc we trust.
-    unsafe {
-        mi_option_set_enabled(mi_option_show_errors, false);
-        mi_option_set_enabled(mi_option_show_stats, false);
-        mi_option_set_enabled(mi_option_verbose, false);
-        mi_register_output(Some(syd_noop_output), std::ptr::null_mut());
-    }
-}
-
 #[inline]
 #[cold]
 fn cold() {}
