@@ -12,7 +12,7 @@ use std::{
     process::{exit, ExitCode},
 };
 
-use libseccomp::{scmp_cmp, ScmpAction, ScmpFilterContext, ScmpSyscall};
+use libseccomp::{scmp_cmp, ScmpAction, ScmpArch, ScmpFilterContext, ScmpSyscall};
 use nix::{
     errno::Errno,
     fcntl::{fcntl, splice, FcntlArg, OFlag, SpliceFFlags},
@@ -35,7 +35,7 @@ use syd::{
     fs::{closeexcept, set_exclusive, set_nonblock},
     ignore_signals,
     io::{winsize_get, winsize_set},
-    ioctl::{TCGETS, TCGETS2, TCSETS, TCSETS2, TIOCGWINSZ, TIOCSWINSZ},
+    ioctl::IoctlMap,
     landlock_policy::LandlockPolicy,
     rng::duprand,
     IgnoreSignalOpts,
@@ -439,23 +439,28 @@ fn confine(sig_fd: RawFd, dry_run: bool, print_rules: bool) -> SydResult<()> {
     // For *2, we use hardcoded versions because not all libcs define them.
     // They are portable as `struct termios2` has the same size
     // across 32-bit and 64-bit architectures.
+    let arch = ScmpArch::native();
+    let ioctl = IoctlMap::new(true);
+    let names = [
+        "TCGETS",
+        "TCSETS",
+        "TCGETS2",
+        "TCSETS2",
+        "TIOCGWINSZ",
+        "TIOCSWINSZ",
+    ];
+    let mut iotty = Vec::with_capacity(names.len());
+    for name in names {
+        if let Some(req) = ioctl.get_value(name, arch) {
+            #[allow(clippy::unnecessary_cast)]
+            iotty.push(req as u64);
+        }
+    }
+    drop(ioctl);
+
     #[expect(clippy::disallowed_methods)]
     let syscall = ScmpSyscall::from_name("ioctl").unwrap();
-    #[expect(clippy::unnecessary_cast)]
-    for request in [
-        *TCGETS,
-        *TCGETS2,
-        *TCSETS,
-        *TCSETS2,
-        *TIOCGWINSZ,
-        *TIOCSWINSZ,
-    ] {
-        let request = if let Some(request) = request {
-            request as u64
-        } else {
-            continue;
-        };
-
+    for request in iotty {
         ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == request)])?;
         if let Some(request) = extend_ioctl(request) {
             ctx.add_rule_conditional(ScmpAction::Allow, syscall, &[scmp_cmp!($arg1 == request)])?;
